# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/mcp_server.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/mcp_server.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from . import options_pb2 as gateway_dot_options__pb2
from . import status_pb2 as gateway_dot_status__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18gateway/mcp_server.proto\x12\x07gateway\x1a\x1b\x62uf/validate/validate.proto\x1a\x15gateway/options.proto\x1a\x14gateway/status.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1bgoogle/api/visibility.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xd7\x07\n\tMcpServer\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12!\n\x0c\x64isplay_name\x18\x02 \x01(\tB\x0b\xe2\x41\x01\x01\xbaH\x04r\x02\x18?\x12!\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x0c\xe2\x41\x01\x01\xbaH\x05r\x03\x18\xa0\x1f\x12\x36\n\x0b\x63reate_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x35\n\x0bupdate_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\x12\x1a\n\x0c\x65ndpoint_url\x18\x06 \x01(\tB\x04\xe2\x41\x01\x01\x12\x1c\n\x0e\x61pi_key_secret\x18\x07 \x01(\tB\x04\xe2\x41\x01\x01\x12\x17\n\tsimulated\x18\x08 \x01(\x08\x42\x04\xe2\x41\x01\x01\x12.\n\x05state\x18\n \x01(\x0e\x32\x18.gateway.McpServer.StateB\x05\xe2\x41\x02\x03\x05\x12&\n\x06status\x18\x0b \x01(\x0b\x32\x0f.gateway.StatusB\x05\xe2\x41\x02\x03\x05\x12\x1b\n\rremote_hosted\x18\x0c \x01(\x08\x42\x04\xe2\x41\x01\x01\x12>\n\x0b\x61nnotations\x18\r \x03(\x0b\x32#.gateway.McpServer.AnnotationsEntryB\x04\xe2\x41\x01\x01\x12\x14\n\x06public\x18\x0e \x01(\x08\x42\x04\xe2\x41\x01\x01\x12H\n\x13\x61uthentication_type\x18\x0f \x01(\x0e\x32%.gateway.McpServer.AuthenticationTypeB\x04\xe2\x41\x01\x01\x12,\n\x08\x66\x65\x61tured\x18\x11 \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"n\n\x12\x41uthenticationType\x12#\n\x1f\x41UTHENTICATION_TYPE_UNSPECIFIED\x10\x00\x12\x08\n\x04OPEN\x10\x01\x12\x0b\n\x07\x41PI_KEY\x10\x02\x12\n\n\x06OAUTH2\x10\x03\x12\x10\n\x0c\x42\x45\x41RER_TOKEN\x10\x04\"R\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\n\n\x06\x41\x43TIVE\x10\x02\x12\n\n\x06\x46\x41ILED\x10\x03\x12\x0c\n\x08\x44\x45LETING\x10\x04:r\xea\x41\x62\n\x1a\x61pi.fireworks.ai/McpServer\x12-accounts/{AccountId}/mcpServers/{McpServerId}*\nmcpServers2\tmcpServer\x82\xf1\x04\t\n\x07\x41\x63\x63ount\"y\n\x16\x43reateMcpServerRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12,\n\nmcp_server\x18\x02 \x01(\x0b\x32\x12.gateway.McpServerB\x04\xe2\x41\x01\x02\x12\x1b\n\rmcp_server_id\x18\x03 \x01(\tB\x04\xe2\x41\x01\x02\"^\n\x13GetMcpServerRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x33\n\tread_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"\xc3\x01\n\x15ListMcpServersRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\x12\x33\n\tread_mask\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"n\n\x16ListMcpServersResponse\x12\'\n\x0bmcp_servers\x18\x01 \x03(\x0b\x32\x12.gateway.McpServer\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"}\n\x16UpdateMcpServerRequest\x12,\n\nmcp_server\x18\x01 \x01(\x0b\x32\x12.gateway.McpServerB\x04\xe2\x41\x01\x02\x12\x35\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\",\n\x16\x44\x65leteMcpServerRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.mcp_server_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_MCPSERVER_ANNOTATIONSENTRY']._loaded_options = None
  _globals['_MCPSERVER_ANNOTATIONSENTRY']._serialized_options = b'8\001'
  _globals['_MCPSERVER'].fields_by_name['name']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_MCPSERVER'].fields_by_name['display_name']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['display_name']._serialized_options = b'\342A\001\001\272H\004r\002\030?'
  _globals['_MCPSERVER'].fields_by_name['description']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['description']._serialized_options = b'\342A\001\001\272H\005r\003\030\240\037'
  _globals['_MCPSERVER'].fields_by_name['create_time']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_MCPSERVER'].fields_by_name['update_time']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_MCPSERVER'].fields_by_name['endpoint_url']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['endpoint_url']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['api_key_secret']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['api_key_secret']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['simulated']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['simulated']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['state']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['state']._serialized_options = b'\342A\002\003\005'
  _globals['_MCPSERVER'].fields_by_name['status']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['status']._serialized_options = b'\342A\002\003\005'
  _globals['_MCPSERVER'].fields_by_name['remote_hosted']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['remote_hosted']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['annotations']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['annotations']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['public']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['public']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['authentication_type']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['authentication_type']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['featured']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['featured']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_MCPSERVER']._loaded_options = None
  _globals['_MCPSERVER']._serialized_options = b'\352Ab\n\032api.fireworks.ai/McpServer\022-accounts/{AccountId}/mcpServers/{McpServerId}*\nmcpServers2\tmcpServer\202\361\004\t\n\007Account'
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['mcp_server']._loaded_options = None
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['mcp_server']._serialized_options = b'\342A\001\002'
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['mcp_server_id']._loaded_options = None
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['mcp_server_id']._serialized_options = b'\342A\001\002'
  _globals['_GETMCPSERVERREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETMCPSERVERREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_GETMCPSERVERREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_GETMCPSERVERREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_UPDATEMCPSERVERREQUEST'].fields_by_name['mcp_server']._loaded_options = None
  _globals['_UPDATEMCPSERVERREQUEST'].fields_by_name['mcp_server']._serialized_options = b'\342A\001\002'
  _globals['_UPDATEMCPSERVERREQUEST'].fields_by_name['update_mask']._loaded_options = None
  _globals['_UPDATEMCPSERVERREQUEST'].fields_by_name['update_mask']._serialized_options = b'\342A\001\001'
  _globals['_DELETEMCPSERVERREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETEMCPSERVERREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_MCPSERVER']._serialized_start=268
  _globals['_MCPSERVER']._serialized_end=1251
  _globals['_MCPSERVER_ANNOTATIONSENTRY']._serialized_start=889
  _globals['_MCPSERVER_ANNOTATIONSENTRY']._serialized_end=939
  _globals['_MCPSERVER_AUTHENTICATIONTYPE']._serialized_start=941
  _globals['_MCPSERVER_AUTHENTICATIONTYPE']._serialized_end=1051
  _globals['_MCPSERVER_STATE']._serialized_start=1053
  _globals['_MCPSERVER_STATE']._serialized_end=1135
  _globals['_CREATEMCPSERVERREQUEST']._serialized_start=1253
  _globals['_CREATEMCPSERVERREQUEST']._serialized_end=1374
  _globals['_GETMCPSERVERREQUEST']._serialized_start=1376
  _globals['_GETMCPSERVERREQUEST']._serialized_end=1470
  _globals['_LISTMCPSERVERSREQUEST']._serialized_start=1473
  _globals['_LISTMCPSERVERSREQUEST']._serialized_end=1668
  _globals['_LISTMCPSERVERSRESPONSE']._serialized_start=1670
  _globals['_LISTMCPSERVERSRESPONSE']._serialized_end=1780
  _globals['_UPDATEMCPSERVERREQUEST']._serialized_start=1782
  _globals['_UPDATEMCPSERVERREQUEST']._serialized_end=1907
  _globals['_DELETEMCPSERVERREQUEST']._serialized_start=1909
  _globals['_DELETEMCPSERVERREQUEST']._serialized_end=1953
# @@protoc_insertion_point(module_scope)
