# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/workload_shape.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/workload_shape.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from . import deployment_pb2 as gateway_dot_deployment__pb2
from . import options_pb2 as gateway_dot_options__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cgateway/workload_shape.proto\x12\x07gateway\x1a\x1b\x62uf/validate/validate.proto\x1a\x18gateway/deployment.proto\x1a\x15gateway/options.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1bgoogle/api/visibility.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xe3\r\n\rWorkloadShape\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\x1a\n\x0c\x64isplay_name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x01\x12\x19\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x36\n\x0b\x63reate_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x35\n\x0bupdate_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\x12+\n\ncreated_by\x18\x06 \x01(\tB\x17\xe2\x41\x02\x03\x05\xfa\xd2\xe4\x93\x02\x0c\x12\nOWNER_ONLY\x12X\n\x0b\x61nnotations\x18\x07 \x03(\x0b\x32\'.gateway.WorkloadShape.AnnotationsEntryB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x19\n\nbase_model\x18\x08 \x01(\tB\x05\xe2\x41\x02\x02\x05\x12&\n\x11\x61\x63\x63\x65lerator_count\x18\t \x01(\x05\x42\x0b\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\x12\x38\n\x10\x61\x63\x63\x65lerator_type\x18\n \x01(\x0e\x32\x18.gateway.AcceleratorTypeB\x04\xe2\x41\x01\x01\x12\x36\n\tprecision\x18\x0b \x01(\x0e\x32\x1d.gateway.Deployment.PrecisionB\x04\xe2\x41\x01\x01\x12\x35\n\nworld_size\x18\x0c \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12:\n\x0fgenerator_count\x18\r \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12H\n\x1b\x64isaggregated_prefill_count\x18\x0e \x01(\x05\x42#\xe2\x41\x01\x01\xbaH\x06\x1a\x04\x18\x10(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12K\n disaggregated_prefill_world_size\x18\x0f \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12<\n\x0emax_batch_size\x18\x10 \x01(\x05\x42$\xe2\x41\x01\x01\xbaH\x07\x1a\x05\x18\x80\x08(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x1f\n\renable_addons\x18\x11 \x01(\x08\x42\x08\xe2\x41\x01\x01\xa8\xf1\x04\x01\x12&\n\x11\x64raft_token_count\x18\x12 \x01(\x05\x42\x0b\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\x12\x19\n\x0b\x64raft_model\x18\x13 \x01(\tB\x04\xe2\x41\x01\x01\x12&\n\x18ngram_speculation_length\x18\x14 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12>\n\x13max_lora_batch_size\x18\x15 \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12@\n\x13kv_cache_memory_pct\x18\x16 \x01(\x05\x42#\xe2\x41\x01\x01\xbaH\x06\x1a\x04\x18Z(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12%\n\x17\x65nable_session_affinity\x18\x17 \x01(\x08\x42\x04\xe2\x41\x01\x01\x12-\n\timage_tag\x18\x18 \x01(\tB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12+\n\x16num_lora_device_cached\x18\x19 \x01(\x05\x42\x0b\xe2\x41\x01\x03\xbaH\x04\x1a\x02(\x00\x12.\n\nextra_args\x18\x1a \x03(\tB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12=\n\x12max_context_length\x18\x1b \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12Y\n\x0c\x65xtra_values\x18\x1c \x03(\x0b\x32\'.gateway.WorkloadShape.ExtraValuesEntryB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12G\n\x06\x65ngine\x18\x1d \x01(\x0e\x32\x1a.gateway.Deployment.EngineB\x1b\xe2\x41\x02\x01\x05\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x45xtraValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01:\x86\x01\xea\x41v\n\x1e\x61pi.fireworks.ai/WorkloadShape\x12\x35\x61\x63\x63ounts/{AccountId}/workloadShapes/{WorkloadShapeId}*\x0eworkloadShapes2\rworkloadShape\x82\xf1\x04\t\n\x07\x41\x63\x63ount\"\xb0\x01\n\x1a\x43reateWorkloadShapeRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x34\n\x0eworkload_shape\x18\x02 \x01(\x0b\x32\x16.gateway.WorkloadShapeB\x04\xe2\x41\x01\x02\x12\x1f\n\x11workload_shape_id\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12%\n\x17\x64isable_size_validation\x18\x04 \x01(\x08\x42\x04\xe2\x41\x01\x01\"b\n\x17GetWorkloadShapeRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x33\n\tread_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"\xc7\x01\n\x19ListWorkloadShapesRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\x12\x33\n\tread_mask\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"z\n\x1aListWorkloadShapesResponse\x12/\n\x0fworkload_shapes\x18\x01 \x03(\x0b\x32\x16.gateway.WorkloadShape\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"\xb0\x01\n\x1aUpdateWorkloadShapeRequest\x12\x34\n\x0eworkload_shape\x18\x01 \x01(\x0b\x32\x16.gateway.WorkloadShapeB\x04\xe2\x41\x01\x02\x12\x35\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\x12%\n\x17\x64isable_size_validation\x18\x03 \x01(\x08\x42\x04\xe2\x41\x01\x01\"0\n\x1a\x44\x65leteWorkloadShapeRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.workload_shape_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_WORKLOADSHAPE_ANNOTATIONSENTRY']._loaded_options = None
  _globals['_WORKLOADSHAPE_ANNOTATIONSENTRY']._serialized_options = b'8\001'
  _globals['_WORKLOADSHAPE_EXTRAVALUESENTRY']._loaded_options = None
  _globals['_WORKLOADSHAPE_EXTRAVALUESENTRY']._serialized_options = b'8\001'
  _globals['_WORKLOADSHAPE'].fields_by_name['name']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_WORKLOADSHAPE'].fields_by_name['display_name']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['display_name']._serialized_options = b'\342A\001\001'
  _globals['_WORKLOADSHAPE'].fields_by_name['description']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['description']._serialized_options = b'\342A\001\001'
  _globals['_WORKLOADSHAPE'].fields_by_name['create_time']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_WORKLOADSHAPE'].fields_by_name['update_time']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_WORKLOADSHAPE'].fields_by_name['created_by']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['created_by']._serialized_options = b'\342A\002\003\005\372\322\344\223\002\014\022\nOWNER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['annotations']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['annotations']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['base_model']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['base_model']._serialized_options = b'\342A\002\002\005'
  _globals['_WORKLOADSHAPE'].fields_by_name['accelerator_count']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['accelerator_count']._serialized_options = b'\342A\001\001\272H\004\032\002(\000'
  _globals['_WORKLOADSHAPE'].fields_by_name['accelerator_type']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['accelerator_type']._serialized_options = b'\342A\001\001'
  _globals['_WORKLOADSHAPE'].fields_by_name['precision']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['precision']._serialized_options = b'\342A\001\001'
  _globals['_WORKLOADSHAPE'].fields_by_name['world_size']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['world_size']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['generator_count']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['generator_count']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['disaggregated_prefill_count']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['disaggregated_prefill_count']._serialized_options = b'\342A\001\001\272H\006\032\004\030\020(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['disaggregated_prefill_world_size']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['disaggregated_prefill_world_size']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['max_batch_size']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['max_batch_size']._serialized_options = b'\342A\001\001\272H\007\032\005\030\200\010(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['enable_addons']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['enable_addons']._serialized_options = b'\342A\001\001\250\361\004\001'
  _globals['_WORKLOADSHAPE'].fields_by_name['draft_token_count']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['draft_token_count']._serialized_options = b'\342A\001\001\272H\004\032\002(\000'
  _globals['_WORKLOADSHAPE'].fields_by_name['draft_model']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['draft_model']._serialized_options = b'\342A\001\001'
  _globals['_WORKLOADSHAPE'].fields_by_name['ngram_speculation_length']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['ngram_speculation_length']._serialized_options = b'\342A\001\001'
  _globals['_WORKLOADSHAPE'].fields_by_name['max_lora_batch_size']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['max_lora_batch_size']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['kv_cache_memory_pct']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['kv_cache_memory_pct']._serialized_options = b'\342A\001\001\272H\006\032\004\030Z(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['enable_session_affinity']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['enable_session_affinity']._serialized_options = b'\342A\001\001'
  _globals['_WORKLOADSHAPE'].fields_by_name['image_tag']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['image_tag']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['num_lora_device_cached']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['num_lora_device_cached']._serialized_options = b'\342A\001\003\272H\004\032\002(\000'
  _globals['_WORKLOADSHAPE'].fields_by_name['extra_args']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['extra_args']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['max_context_length']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['max_context_length']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['extra_values']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['extra_values']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE'].fields_by_name['engine']._loaded_options = None
  _globals['_WORKLOADSHAPE'].fields_by_name['engine']._serialized_options = b'\342A\002\001\005\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOADSHAPE']._loaded_options = None
  _globals['_WORKLOADSHAPE']._serialized_options = b'\352Av\n\036api.fireworks.ai/WorkloadShape\0225accounts/{AccountId}/workloadShapes/{WorkloadShapeId}*\016workloadShapes2\rworkloadShape\202\361\004\t\n\007Account'
  _globals['_CREATEWORKLOADSHAPEREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATEWORKLOADSHAPEREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATEWORKLOADSHAPEREQUEST'].fields_by_name['workload_shape']._loaded_options = None
  _globals['_CREATEWORKLOADSHAPEREQUEST'].fields_by_name['workload_shape']._serialized_options = b'\342A\001\002'
  _globals['_CREATEWORKLOADSHAPEREQUEST'].fields_by_name['workload_shape_id']._loaded_options = None
  _globals['_CREATEWORKLOADSHAPEREQUEST'].fields_by_name['workload_shape_id']._serialized_options = b'\342A\001\001'
  _globals['_CREATEWORKLOADSHAPEREQUEST'].fields_by_name['disable_size_validation']._loaded_options = None
  _globals['_CREATEWORKLOADSHAPEREQUEST'].fields_by_name['disable_size_validation']._serialized_options = b'\342A\001\001'
  _globals['_GETWORKLOADSHAPEREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETWORKLOADSHAPEREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_GETWORKLOADSHAPEREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_GETWORKLOADSHAPEREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_LISTWORKLOADSHAPESREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_UPDATEWORKLOADSHAPEREQUEST'].fields_by_name['workload_shape']._loaded_options = None
  _globals['_UPDATEWORKLOADSHAPEREQUEST'].fields_by_name['workload_shape']._serialized_options = b'\342A\001\002'
  _globals['_UPDATEWORKLOADSHAPEREQUEST'].fields_by_name['update_mask']._loaded_options = None
  _globals['_UPDATEWORKLOADSHAPEREQUEST'].fields_by_name['update_mask']._serialized_options = b'\342A\001\001'
  _globals['_UPDATEWORKLOADSHAPEREQUEST'].fields_by_name['disable_size_validation']._loaded_options = None
  _globals['_UPDATEWORKLOADSHAPEREQUEST'].fields_by_name['disable_size_validation']._serialized_options = b'\342A\001\001'
  _globals['_DELETEWORKLOADSHAPEREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETEWORKLOADSHAPEREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_WORKLOADSHAPE']._serialized_start=276
  _globals['_WORKLOADSHAPE']._serialized_end=2039
  _globals['_WORKLOADSHAPE_ANNOTATIONSENTRY']._serialized_start=1800
  _globals['_WORKLOADSHAPE_ANNOTATIONSENTRY']._serialized_end=1850
  _globals['_WORKLOADSHAPE_EXTRAVALUESENTRY']._serialized_start=1852
  _globals['_WORKLOADSHAPE_EXTRAVALUESENTRY']._serialized_end=1902
  _globals['_CREATEWORKLOADSHAPEREQUEST']._serialized_start=2042
  _globals['_CREATEWORKLOADSHAPEREQUEST']._serialized_end=2218
  _globals['_GETWORKLOADSHAPEREQUEST']._serialized_start=2220
  _globals['_GETWORKLOADSHAPEREQUEST']._serialized_end=2318
  _globals['_LISTWORKLOADSHAPESREQUEST']._serialized_start=2321
  _globals['_LISTWORKLOADSHAPESREQUEST']._serialized_end=2520
  _globals['_LISTWORKLOADSHAPESRESPONSE']._serialized_start=2522
  _globals['_LISTWORKLOADSHAPESRESPONSE']._serialized_end=2644
  _globals['_UPDATEWORKLOADSHAPEREQUEST']._serialized_start=2647
  _globals['_UPDATEWORKLOADSHAPEREQUEST']._serialized_end=2823
  _globals['_DELETEWORKLOADSHAPEREQUEST']._serialized_start=2825
  _globals['_DELETEWORKLOADSHAPEREQUEST']._serialized_end=2873
# @@protoc_insertion_point(module_scope)
