# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/evaluation_job.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/evaluation_job.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import deployment_pb2 as gateway_dot_deployment__pb2
from . import job_progress_pb2 as gateway_dot_job__progress__pb2
from . import options_pb2 as gateway_dot_options__pb2
from . import status_pb2 as gateway_dot_status__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cgateway/evaluation_job.proto\x12\x07gateway\x1a\x18gateway/deployment.proto\x1a\x1agateway/job_progress.proto\x1a\x15gateway/options.proto\x1a\x14gateway/status.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1bgoogle/api/visibility.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x91\x07\n\rEvaluationJob\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\x1a\n\x0c\x64isplay_name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x01\x12\x36\n\x0b\x63reate_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x19\n\ncreated_by\x18\x04 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\'\n\x05state\x18\x05 \x01(\x0e\x32\x11.gateway.JobStateB\x05\xe2\x41\x02\x03\x05\x12&\n\x06status\x18\x06 \x01(\x0b\x32\x0f.gateway.StatusB\x05\xe2\x41\x02\x03\x05\x12\x17\n\tevaluator\x18\x07 \x01(\tB\x04\xe2\x41\x01\x02\x12\x1b\n\rinput_dataset\x18\x08 \x01(\tB\x04\xe2\x41\x01\x02\x12\x1c\n\x0eoutput_dataset\x18\n \x01(\tB\x04\xe2\x41\x01\x02\x12>\n\x07metrics\x18\t \x03(\x0b\x32#.gateway.EvaluationJob.MetricsEntryB\x08\xe2\x41\x01\x03\xa0\xf1\x04\x01\x12\x1a\n\x0coutput_stats\x18\x0c \x01(\tB\x04\xe2\x41\x01\x05\x12\x35\n\x0bupdate_time\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\x12\x46\n\"reinforcement_fine_tuning_epoch_id\x18\r \x01(\tB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12;\n\x17skip_dataset_validation\x18\x0e \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12G\n\x0cjob_progress\x18\x0f \x01(\x0b\x32\x14.gateway.JobProgressB\x1b\xe2\x41\x02\x03\x05\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12;\n\x06region\x18\x10 \x01(\x0e\x32\x0f.gateway.RegionB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x1a.\n\x0cMetricsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01:\x88\x01\xea\x41v\n\x1e\x61pi.fireworks.ai/EvaluationJob\x12\x35\x61\x63\x63ounts/{AccountId}/evaluationJobs/{EvaluationJobId}*\x0e\x45valuationJobs2\rEvaluationJob\x82\xf1\x04\x0b\n\x07\x41\x63\x63ount\x18\x01\"b\n\x17GetEvaluationJobRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x33\n\tread_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"\x89\x01\n\x1a\x43reateEvaluationJobRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x34\n\x0e\x65valuation_job\x18\x02 \x01(\x0b\x32\x16.gateway.EvaluationJobB\x04\xe2\x41\x01\x02\x12\x1f\n\x11\x65valuation_job_id\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\"\xfa\x01\n\x19ListEvaluationJobsRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\x12\x31\n\rshow_internal\x18\x06 \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x33\n\tread_mask\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"z\n\x1aListEvaluationJobsResponse\x12/\n\x0f\x65valuation_jobs\x18\x01 \x03(\x0b\x32\x16.gateway.EvaluationJob\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"0\n\x1a\x44\x65leteEvaluationJobRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.evaluation_job_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_EVALUATIONJOB_METRICSENTRY']._loaded_options = None
  _globals['_EVALUATIONJOB_METRICSENTRY']._serialized_options = b'8\001'
  _globals['_EVALUATIONJOB'].fields_by_name['name']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_EVALUATIONJOB'].fields_by_name['display_name']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['display_name']._serialized_options = b'\342A\001\001'
  _globals['_EVALUATIONJOB'].fields_by_name['create_time']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_EVALUATIONJOB'].fields_by_name['created_by']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['created_by']._serialized_options = b'\342A\002\003\005'
  _globals['_EVALUATIONJOB'].fields_by_name['state']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['state']._serialized_options = b'\342A\002\003\005'
  _globals['_EVALUATIONJOB'].fields_by_name['status']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['status']._serialized_options = b'\342A\002\003\005'
  _globals['_EVALUATIONJOB'].fields_by_name['evaluator']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['evaluator']._serialized_options = b'\342A\001\002'
  _globals['_EVALUATIONJOB'].fields_by_name['input_dataset']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['input_dataset']._serialized_options = b'\342A\001\002'
  _globals['_EVALUATIONJOB'].fields_by_name['output_dataset']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['output_dataset']._serialized_options = b'\342A\001\002'
  _globals['_EVALUATIONJOB'].fields_by_name['metrics']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['metrics']._serialized_options = b'\342A\001\003\240\361\004\001'
  _globals['_EVALUATIONJOB'].fields_by_name['output_stats']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['output_stats']._serialized_options = b'\342A\001\005'
  _globals['_EVALUATIONJOB'].fields_by_name['update_time']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_EVALUATIONJOB'].fields_by_name['reinforcement_fine_tuning_epoch_id']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['reinforcement_fine_tuning_epoch_id']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_EVALUATIONJOB'].fields_by_name['skip_dataset_validation']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['skip_dataset_validation']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_EVALUATIONJOB'].fields_by_name['job_progress']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['job_progress']._serialized_options = b'\342A\002\003\005\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_EVALUATIONJOB'].fields_by_name['region']._loaded_options = None
  _globals['_EVALUATIONJOB'].fields_by_name['region']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_EVALUATIONJOB']._loaded_options = None
  _globals['_EVALUATIONJOB']._serialized_options = b'\352Av\n\036api.fireworks.ai/EvaluationJob\0225accounts/{AccountId}/evaluationJobs/{EvaluationJobId}*\016EvaluationJobs2\rEvaluationJob\202\361\004\013\n\007Account\030\001'
  _globals['_GETEVALUATIONJOBREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETEVALUATIONJOBREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_GETEVALUATIONJOBREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_GETEVALUATIONJOBREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_CREATEEVALUATIONJOBREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATEEVALUATIONJOBREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATEEVALUATIONJOBREQUEST'].fields_by_name['evaluation_job']._loaded_options = None
  _globals['_CREATEEVALUATIONJOBREQUEST'].fields_by_name['evaluation_job']._serialized_options = b'\342A\001\002'
  _globals['_CREATEEVALUATIONJOBREQUEST'].fields_by_name['evaluation_job_id']._loaded_options = None
  _globals['_CREATEEVALUATIONJOBREQUEST'].fields_by_name['evaluation_job_id']._serialized_options = b'\342A\001\001'
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['show_internal']._loaded_options = None
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['show_internal']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_LISTEVALUATIONJOBSREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_DELETEEVALUATIONJOBREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETEEVALUATIONJOBREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_EVALUATIONJOB']._serialized_start=297
  _globals['_EVALUATIONJOB']._serialized_end=1210
  _globals['_EVALUATIONJOB_METRICSENTRY']._serialized_start=1025
  _globals['_EVALUATIONJOB_METRICSENTRY']._serialized_end=1071
  _globals['_GETEVALUATIONJOBREQUEST']._serialized_start=1212
  _globals['_GETEVALUATIONJOBREQUEST']._serialized_end=1310
  _globals['_CREATEEVALUATIONJOBREQUEST']._serialized_start=1313
  _globals['_CREATEEVALUATIONJOBREQUEST']._serialized_end=1450
  _globals['_LISTEVALUATIONJOBSREQUEST']._serialized_start=1453
  _globals['_LISTEVALUATIONJOBSREQUEST']._serialized_end=1703
  _globals['_LISTEVALUATIONJOBSRESPONSE']._serialized_start=1705
  _globals['_LISTEVALUATIONJOBSRESPONSE']._serialized_end=1827
  _globals['_DELETEEVALUATIONJOBREQUEST']._serialized_start=1829
  _globals['_DELETEEVALUATIONJOBREQUEST']._serialized_end=1877
# @@protoc_insertion_point(module_scope)
